@@ -1,31 +1,86 @@
+/**
+ * @file   m_options.c
+ * @author Akinobu Lee
+ * @date   Thu May 12 18:52:07 2005
+ * 
+ * <JA>
+ * @brief  jconf䥳ޥɤΥץ롥
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Process options and configurations from jconf or command argument.
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2004 Kyoto University
- * Copyright (c) 2000-2004 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* m_options.c --- process options and set values */
-
-/* $Id: m_options.c,v 1.23 2004/04/27 04:55:31 ri Exp $ */
-
 #include <julius.h>
 
-/* change relative path to absolute path */
-static char *
+/** 
+ * <JA>
+ * @brief  ХѥեѥѴ롥
+ * 
+ * եΥѥ̾ХѥǤСȥǥ쥯ȥĤ
+ * եѥѴ֤ХѥǤСΤޤ֤
+ * 
+ * @param filename [in] եΥѥ̾
+ * @param dirname [in] ȥǥ쥯ȥΥѥ̾
+ * 
+ * @return Хѥ̾ä˳դ줿Хåե
+ * </JA>
+ * <EN>
+ * @brief  Change relative path to full path.
+ *
+ * If the file path is given as relative, prepend the dirname to it.
+ * If the file path is full, just copy it to new buffer and return.
+ * 
+ * @param filename [in] file path name
+ * @param dirname [in] full path of current directory
+ * 
+ * @return newly malloced buffer holding the full path name.
+ * </EN>
+ */
+char *
 filepath(char *filename, char *dirname)
 {
   char *p;
-  if (dirname != NULL && filename[0] != '/') {
+  if (dirname != NULL && filename[0] != '/'
+#if defined(_WIN32)
+      && filename[0] != '\\' && !(strlen(filename) >= 3 && filename[1] == ':')
+#endif
+      ) {
     p = (char *)mymalloc(strlen(filename) + strlen(dirname) + 1);
     strcpy(p, dirname);
     strcat(p, filename);
   } else {
-    p = filename;
+    p = strcpy((char *)mymalloc(strlen(filename)+1), filename);
   }
   return p;
 }
 
-/* error exit when required argument was not specified */
+/** 
+ * <JA>
+ * ɬפʰͿʤä˥顼λ롥
+ * 
+ * @param opt [in] ץʸ
+ * 
+ * @return NULL.
+ * </JA>
+ * <EN>
+ * Error exit when required argument was not specified.
+ * 
+ * @param opt [in] option string
+ * 
+ * @return NULL.
+ * </EN>
+ */
 static char *
 args_needed_exit(char *opt)
 {
@@ -34,7 +89,22 @@ args_needed_exit(char *opt)
   return(NULL);
 }
 
-/* parse options and set values */
+/** 
+ * <JA>
+ * @brief  ץϤбͤ򥻥åȤ롥
+ *
+ * @param argc [in] @a argv ˴ޤޤο
+ * @param argv [in] ɽʸ
+ * @param cwd [in] ȥǥ쥯ȥʸ
+ * </JA>
+ * <EN>
+ * Parse options and set values.
+ * 
+ * @param argc [in] number of elements in @a argv
+ * @param argv [in] array of argument strings
+ * @param cwd [in] current directory string
+ * </EN>
+ */
 void
 opt_parse(int argc, char *argv[], char *cwd)
 {
@@ -175,10 +245,21 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-lookuprange")) { /* trellis neighbor range */
       lookup_range = atoi(NEXTARG);
       continue;
+#ifdef GRAPHOUT
+    } else if (strmatch(argv[i],"-graphrange")) { /* neighbor merge range frame */
+      graph_merge_neighbor_range = atoi(NEXTARG);
+      continue;
+#endif
 #ifdef USE_DFA
     } else if (strmatch(argv[i],"-looktrellis")) { /* activate loopuprange */
       looktrellis_flag = TRUE;
       continue;
+    } else if (strmatch(argv[i],"-multigramout")) { /* enable per-grammar decoding on 2nd pass */
+      multigramout_flag = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-nomultigramout")) { /* disable per-grammar decoding on 2nd pass */
+      multigramout_flag = FALSE;
+      continue;
 #endif
 #ifdef CATEGORY_TREE
     } else if (strmatch(argv[i],"-oldtree")) { /* use old tree function */
@@ -269,6 +350,37 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-proginterval")) { /* interval for -progout */
       progout_interval = atoi(NEXTARG);
       continue;
+    } else if (strmatch(argv[i],"-nocharconv")) { /* disable character set conversion */
+#ifdef CHARACTER_CONVERSION
+      from_code = NULL;
+      to_code = NULL;
+#else
+      j_printerr("Warning: character set conversion disabled, option \"-nocharconv\" ignored\n");
+#endif
+      continue;
+    } else if (strmatch(argv[i],"-charconv")) {	/* enable character set conversion */
+#ifdef CHARACTER_CONVERSION
+      from_code = NEXTARG;
+      to_code = NEXTARG;
+#else
+      j_printerr("Warning: character set conversion disabled, option \"-charconv\" ignored\n");
+      i+=2;
+#endif
+      continue;
+    } else if (strmatch(argv[i],"-kanji")) {
+#ifdef CHARACTER_CONVERSION
+      from_code = NULL;
+      tmparg = NEXTARG;
+      if (strmatch(tmparg, "noconv")) {
+	to_code = NULL;
+      } else {
+	to_code = tmparg;
+      }
+#else
+      j_printerr("Warning: character set conversion disabled, option \"-kanji\" ignored\n");
+      i++;
+#endif
+      continue;
     } else if (strmatch(argv[i],"-demo")) { /* quiet + progout */
       debug2_flag = verbose_flag = FALSE;
       progout_flag = TRUE;
@@ -314,6 +426,23 @@ opt_parse(int argc, char *argv[], char *cwd)
       lm_penalty_trans = (LOGPROB)atof(NEXTARG);
       continue;
 #else  /* USE_DFA */
+    } else if (strmatch(argv[i],"-gram")) { /* comma-separatedlist of grammar prefix */
+      multigram_add_prefix_list(NEXTARG, cwd);
+      continue;
+    } else if (strmatch(argv[i],"-gramlist")) { /* file of grammar prefix list */
+      multigram_add_prefix_filelist(filepath(NEXTARG, cwd));
+      continue;
+    } else if (strmatch(argv[i],"-nogram")) { /* remove grammar list */
+      multigram_remove_gramlist();
+      if (dfa_filename != NULL) {
+	free(dfa_filename);
+	dfa_filename = NULL;
+      }
+      if (dictfilename != NULL) {
+	free(dictfilename);
+	dictfilename = NULL;
+      }
+      continue;
     } else if (strmatch(argv[i],"-dfa")) { /* DFA filename */
       dfa_filename = filepath(NEXTARG, cwd);
       continue;
@@ -327,6 +456,14 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-spmodel") || strmatch(argv[i], "-sp")) { /* name of short pause word */
       spmodel_name = NEXTARG;
       continue;
+#ifdef MULTIPATH_VERSION
+    } else if (strmatch(argv[i],"-iwsp")) { /* enable inter-word short pause handing */
+      enable_iwsp = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-iwsppenalty")) { /* set inter-word short pause transition penalty */
+      iwsp_penalty = atof(NEXTARG);
+      continue;
+#endif
 #ifdef USE_NGRAM
     } else if (strmatch(argv[i],"-silhead")) { /* head silence word name */
       head_silname = NEXTARG;
@@ -354,6 +491,7 @@ opt_parse(int argc, char *argv[], char *cwd)
       separate_wnum = atoi(NEXTARG);
 #else
       j_printerr("Warning: option \"-sepnum\" ignored\n");
+      i++;
 #endif
       continue;
 #endif /* USE_NGRAM */
@@ -473,7 +611,28 @@ opt_parse(int argc, char *argv[], char *cwd)
       cm_alpha = (LOGPROB)atof(NEXTARG);
 #endif
       continue;
+#ifdef CM_SEARCH_LIMIT
+    } else if (strmatch(argv[i],"-cmthres")) { /* CM cut threshold for CM decoding */
+      cm_cut_thres = (LOGPROB)atof(NEXTARG);
+      continue;
+#endif
+#ifdef CM_SEARCH_LIMIT_POP
+    } else if (strmatch(argv[i],"-cmthres2")) { /* CM cut threshold for CM decoding */
+      cm_cut_thres_pop = (LOGPROB)atof(NEXTARG);
+      continue;
+#endif
 #endif /* CONFIDENCE_MEASURE */
+    } else if (strmatch(argv[i],"-gmm")) { /* load SS parameter from file */
+      gmm_filename = filepath(NEXTARG, cwd);
+      continue;
+    } else if (strmatch(argv[i],"-gmmnum")) { /* num of Gaussian pruning for GMM */
+      gmm_gprune_num = atoi(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-gmmreject")) {
+      tmparg = NEXTARG;
+      gmm_reject_cmn_string = (char *)mymalloc(strlen(tmparg)+1);
+      strcpy(gmm_reject_cmn_string, tmparg);
+      continue;
     } else if (strmatch(argv[i],"-help")) { /* output version and option */
       detailed_help();
     } else if (strmatch(argv[i],"--help")) {
@@ -496,7 +655,7 @@ opt_parse(int argc, char *argv[], char *cwd)
 	break;
 #endif
       case 'b':			/* beam width in 1st pass */
-	trellis_beam_width = atoi(NEXTARG);
+	specified_trellis_beam_width = atoi(NEXTARG);
 	break;
       case 's':			/* stack size in 2nd pass */
 	stack_size = atoi(NEXTARG);
