@@ -1,26 +1,48 @@
+/**
+ * @file   main.c
+ * @author Akinobu Lee
+ * @date   Wed May 18 15:02:55 2005
+ * 
+ * <JA>
+ * @brief  Julius/Julian ᥤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Main function of Julius/Julian
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2004 Kyoto University
- * Copyright (c) 1997-2000 IPA, Japan
- * Copyright (c) 2000-2004 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1997-2000 Information-technology Promotion Agency, Japan
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* main.c --- main routine */
-
-/* $Id: main.c,v 1.20 2004/04/27 04:55:31 ri Exp $ */
-
-#define GLOBAL_VARIABLE_DEFINE	/* actually make global vars in global.h */
+#define GLOBAL_VARIABLE_DEFINE	///< Actually make global vars in global.h
 #include <julius.h>
 #include <signal.h>
-#include <unistd.h>
-#include <errno.h>
+#if defined(_WIN32) && !defined(__CYGWIN32__)
+#include <mbctype.h>
+#include <mbstring.h>
+#endif
 
-HTK_Param *param;		/* parameter vector */
 
 /* ---------- utility functions -----------------------------------------*/
 #ifdef REPORT_MEMORY_USAGE
-/* output memory usage (virtual / resident) by invoking "ps" */
-/* for linux, sol2  */
+/** 
+ * <JA>
+ * ̾ｪλ˻ѥ̤Ĵ٤ƽϤ (Linux, sol2
+ * 
+ * </JA>
+ * <EN>
+ * Get process size and output on normal exit. (Linux, sol2)
+ * 
+ * </EN>
+ */
 static void
 print_mem()
 {
@@ -34,15 +56,49 @@ print_mem()
 	  
 
 /* --------------------- speech buffering ------------------ */
-/* callback function for adin_go: store each segmented speech data to buffer */
-/* will be used when NOT realtime mode */
+/**
+ * Temporal buffer to save the recorded-but-unprocessed samples
+ * when the length of a speech segment exceeds the limit
+ * (i.e. MAXSPEECHLEN samples).  They will be restored on the
+ * next input at the top of the recording buffer.
+ * 
+ */
 static SP16 *overflowed_samples = NULL;
+/**
+ * Length of above.
+ * 
+ */
 static int overflowed_samplenum;
-/* return value:
-   -1 ... error -> exit, terminate 
-   0  ... continue reading
-   1  ... segment detected -> force adin_go to segment (do not flush the cycle buffer)
-*/
+/** 
+ * <JA>
+ * @brief  ж֤βǡХåե¸뤿 adin_go() callback
+ *
+ * δؿϡФ줿Ϥ༡Хåե @a speech ˵Ͽ
+ * ޤХåե⡼ɡʡꥢ륿⡼ɡˤǧԤʤ
+ * ȤѤޤ
+ * 
+ * @param now [in] Ф줿ȷǡ
+ * @param len [in] @a now Ĺ(ץ)
+ * 
+ * @return 顼 -1 (adin_go ¨Ǥ)̾ 0 (adin_go 
+ * ³Ԥ)ֽλ׵ 1 (adin_go ϸߤβ֤Ĥ)
+ * 
+ * </JA>
+ * <EN>
+ * @brief  adin_go() callback to score each detected speech segment to buffer.
+ *
+ * This function records the incomping speech segments detected in adin_go()
+ * to a buffer @a speech.  This function will be used when recognition runs
+ * in buffered mode (= non-realtime mode).
+ * 
+ * @param now [in] input speech samples.
+ * @param len [in] length of @a now in samples
+ * 
+ * @return -1 on error (tell adin_go() to terminate), 0 on success (tell
+ * adin_go() to continue recording), or 1 when this function requires
+ * input segmentation.
+ * </EN>
+ */
 int
 adin_cut_callback_store_buffer(SP16 *now, int len)
 {
@@ -107,13 +163,46 @@ adin_cut_callback_store_buffer(SP16 *now, int len)
 }
 
 /* return malloced next file name from inputlist (return NULL on error) */
+/** 
+ * <JA>
+ * @brief  ϥեꥹȤϥե̾1ɤ߹֤
+ *
+ * δؿ MFCC եϻϥꥹȥե inputlist_filename
+ * ϥե̾1ɤ߹ߡͤ򿷤 malloc 줿Хåե
+ * ˳Ǽ֤ޤԤ "#" ǻϤޤԤ̵뤵ޤ
+ *
+ * δؿ MFCC եϻ (-input mfcfile) ΤȤΤѤޤ
+ * եϤξϥեꥹȤ adin_go() ǰޤ
+ * 
+ * @return ϥե̾Ǽ줿 malloc 줿Хåե
+ * Υݥ󥿡뤤ϼϥե̾ʤСEOF ãNULL
+ * </JA>
+ * <EN>
+ * @brief  Get the next input filename from input list file and return
+ * the filename.
+ *
+ * This function is used when "-input mfcfile" and "-filelist" is both
+ * specified, to get input filenames successively from a given input list
+ * file "inputlist_filename".  When called, the next filename will be
+ * read from the list file
+ * and stored in a newly malloced buffer.  Blank lines or lines starting
+ * with "#" in the list file will be discarded.
+ *
+ * This function will only be used when "-filelist" is used when MFCC
+ * input is specified ("-input mfcfile").  When speech waveform input,
+ * functions in adin_go() will handles the list file instead of this
+ * function.
+ * 
+ * @return pointer to a newly malloced buffer that holds the next input
+ * filename, or NULL when no next file availbale (the list file reached
+ * EOF).
+ * </EN>
+ */
 static char *
 mfcfilelist_nextfile()
 {
-  static FILE *mfclist = NULL;	/* filelist of MFC file (-filelist) */
+  static FILE *mfclist = NULL;	/* current filelist of MFC file (-filelist) */
   static char *buf;
-  int newline;
-  /* filelist of rawfile is handled within adin_go() */
   
   if (mfclist == NULL) {	/* not opened yet */
     if ((mfclist = fopen(inputlist_filename, "r")) == NULL) { /* open error */
@@ -121,15 +210,14 @@ mfcfilelist_nextfile()
     }
   }
   buf = mymalloc(MAXLINELEN);
-  while(fgets(buf, MAXLINELEN, mfclist) != NULL) {
-    newline = strlen(buf) - 1;
-    if (buf[newline] == '\n') buf[newline] = '\0'; /* chop newline character */
+  while(getl_fp(buf, MAXLINELEN, mfclist) != NULL) {
     if (buf[0] == '\0') continue; /* skip blank line */
     if (buf[0] == '#') continue; /* skip commented-out line */
     /* read a filename */
     return buf;
   }
   /* end of inputfile list */
+  free(buf);
   fclose(mfclist);
   mfclist = NULL;
   return NULL;
@@ -139,10 +227,33 @@ mfcfilelist_nextfile()
 /* ------------------------------------------------------------------- */
 /* ------------- Main Recognition Loop ------------------------------- */
 /* ------------------------------------------------------------------- */
+
+/** 
+ * <JA>
+ * @brief  ᥤβǧ¹ԥ롼
+ *
+ * δؿϲǧ¹Ԥᥤ롼פǤǽ˥ƥν
+ * Ԥʤθ奤٥ȥ롼פޤ٥ȥ롼פǤϲ֤
+ * ȡβ֤ˤĤǧ1ѥ2ѥԤʤ̤
+ * Ϥޤޤ⥸塼⡼ɤξ̿ᥳޥɤνθƤӽФ
+ * Ԥʤޤ
+ * </JA>
+ * <EN>
+ * @brief  Main recognition process loop.
+ *
+ * This function is a main loop to execute recognition.  First the whole
+ * recognition system is setup, and then it enters a main event loop.
+ * If the start of speech input is detected, the recognition process of the
+ * first pass and second pass will be performed in turn, and the result will
+ * be output, and return to the event loop.  In module mode, the module
+ * commands from module client will also be received and processed here.
+ * </EN>
+ */
 void
 main_recognition_loop()
 {
   char *speechfilename;	/* pathname of speech file or MFCC file */
+  HTK_Param *param = NULL;		/* parameter vector */
   HTK_Param *selected_param;
   int ret;
   int file_counter;
@@ -167,7 +278,7 @@ main_recognition_loop()
   /* Visualize: initialize GTK */
   visual_init();
 #endif
-  
+
   /***************************/
   /***************************/
   /** Main Recognition Loop **/
@@ -220,6 +331,9 @@ main_recognition_loop()
       /* count number of processed files */
       file_counter++;
     } else {			/* raw speech input */
+      /********************************/
+      /* ready to read waveform input */
+      /********************************/
       VERMES("### read waveform input\n");
       /* begin A/D input */
       if (adin_begin() == FALSE) {
@@ -239,15 +353,26 @@ main_recognition_loop()
       }
     }
     
-    /*********************************/
-    /* do recognition for each input */
-    /*********************************/
+#ifdef USE_DFA
+    /* if no grammar specified on startup, start with pause status */
+    if (module_mode) {
+      if (dfa == NULL || winfo == NULL) { /* stop when no grammar found */
+	msock_exec_command("PAUSE");
+      }
+    }
+#endif
+  
+    /******************************************************************/
+    /* do recognition for each incoming segment from the input stream */
+    /******************************************************************/
     while (1) {
 
     start_recog:
 
       if (module_mode) {
-	/* process module command */
+	/*****************************/
+	/* module command processing */
+	/*****************************/
 	/* If recognition is running (active), commands are polled only once
 	   here, and if any, process the command, and continue the recognition.
 	   If recognition is sleeping (inactive), wait here for any command to
@@ -266,7 +391,9 @@ main_recognition_loop()
 	  msock_process_command();
 	}
 #ifdef USE_DFA
-	/* check for grammar to change */
+	/*********************************************************/
+	/* check for grammar to change, and rebuild if necessary */
+	/*********************************************************/
 	multigram_exec();
 	if (dfa == NULL || winfo == NULL) { /* stop when no grammar found */
 	  msock_exec_command("PAUSE");
@@ -283,24 +410,34 @@ main_recognition_loop()
 	/* and jump to the start point of 1st pass */
 	ret = 0;
       } else {
-	/***********************/
-	/* raw wave data input */
-	/***********************/
+	/****************************************************/
+	/* raw wave data input (mic, file, adinnet, etc...) */
+	/****************************************************/
 	if (realtime_flag) {
-	  /****************************/
-	  /* SPECIAL: realtime 1st-pass */
-	  /****************************/
-	  /* store, analysis and search in a pipeline (realtime_1stpass.c) */
+	  /********************************************/
+	  /* REALTIME ON-THE-FLY DECODING OF 1ST-PASS */
+	  /********************************************/
+	  /* store, analysis and search in a pipeline  */
+	  /* main function is RealTimePipeLine() at realtime-1stpass.c, and
+	     it will be periodically called for each incoming input segment
+	     from the AD-in function adin_go().  RealTimePipeLine() will be
+	     called as a callback function from adin_go() */
 	  /* after this part, directly jump to the beginning of the 2nd pass */
 #ifdef SP_BREAK_CURRENT_FRAME
 	  if (rest_param) {
+	    /*****************************************************************/
+	    /* short-pause segmentation: process last remaining frames first */
+	    /*****************************************************************/
 	    /* last was segmented by short pause */
 	    /* the margin segment in the last input should be re-processed first */
+	    /* process the last remaining parameters */
 	    ret = RealTimeResume();
 	    if (ret < 0) {		/* error end in the margin */
 	      j_error("error in resuming last fragment\n"); /* exit now! */
 	    }
 	    if (ret != 1) {	/* if segmented again in the margin, not process the rest */
+	      /* last parameters has been processed, so continue with the
+		 current input as normal */
 	      /* output listening start message */
 	      status_recready();
 	      if (module_mode) {
@@ -312,7 +449,9 @@ main_recognition_loop()
 	      }
 	      if (ret < 0) {		/* error end in adin_go */
 		if (module_mode && ret == -2) {	/* terminated by module */
-		  goto start_recog;
+		  RealTimeTerminate();
+		  param = NULL;
+		  goto end_recog; /* cancel this recognition */
 		}
 		j_error("error in adin_go\n");          /* exit now! */
 	      }
@@ -321,8 +460,15 @@ main_recognition_loop()
 	  } else {
 	    /* last was not segmented, process the incoming input  */
 #endif
+	    /**********************************/
+	    /* process incoming speech stream */
+	    /**********************************/
+	    /* end of this input will be determined by either end of stream
+	       (in case of file input), or silence detection by adin_go(), or
+	       'TERMINATE' command from module (if module mode) */
+	    /* prepare work area for on-the-fly processing */
 	    RealTimePipeLinePrepare();
-	    /* output listening start message */
+	    /* output 'listening start' message */
 	    status_recready();
 	    /* process the incoming input */
 	    if (module_mode) {
@@ -332,13 +478,18 @@ main_recognition_loop()
 	    }
 	    if (ret < 0) {		/* error end in adin_go */
 	      if (module_mode && ret == -2) {	/* terminated by module */
-		goto start_recog;
+		RealTimeTerminate();
+		param = NULL;
+		goto end_recog;
 	      }
 	      j_error("error in adin_go\n");            /* exit now! */
 	    }
 #ifdef SP_BREAK_CURRENT_FRAME
 	  }
 #endif
+	  /******************************************************************/
+	  /* speech stream has been processed on-the-fly, and 1st pass ends */
+	  /******************************************************************/
 	  /* last procedure of 1st-pass */
 	  param = RealTimeParam(&backmax);
 	  /* output stopped recording message */
@@ -352,60 +503,67 @@ main_recognition_loop()
 	  /* end of 1st pass, jump to 2nd pass */
 	  goto end_1pass;
 	  
-	}
+	} /* end of realtime_flag && speech stream input */
 	
-	/********************************/
-	/* store raw samples & analysis */
-	/********************************/
+	/******************************************/
+	/* buffered speech input (not on-the-fly) */
+	/******************************************/
 #ifdef SP_BREAK_CURRENT_FRAME
 	if (rest_param == NULL) { /* no segment left */
 #endif
+	  /****************************************/
+	  /* store raw speech samples to speech[] */
+	  /****************************************/
 	  speechlen = 0;
+	  param = NULL;
 	  /* if needed, begin recording the incoming speech data to a file */
 	  if (record_dirname != NULL) {
 	    record_sample_open();
 	  }
-	  /* output listening start message */
+	  /* output 'listening start' message */
 	  status_recready();
 	  if (module_mode) {
 	    /* tell module to start recording */
+	    /* the "adin_cut_callback_store_buffer" simply stores
+	       the input speech to a buffer "speech[]" */
+	    /* end of this input will be determined by either end of stream
+	       (in case of file input), or silence detection by adin_go(), or
+	       'TERMINATE' command from module (if module mode) */
 	    ret = adin_go(adin_cut_callback_store_buffer, msock_check_in_adin);
 	  } else {
 	    ret = adin_go(adin_cut_callback_store_buffer, NULL);
 	  }
 	  if (ret < 0) {		/* error end in adin_go */
 	    if (module_mode && ret == -2) {	/* terminated by module */
-	      goto start_recog;
+	      goto end_recog;
 	    }
 	    j_error("error in adin_go\n");              /* exit now! */
 	  }
 	  /* output stopped recording message */
 	  status_recend();
 
+	  /* output recorded length */
 	  seclen = (float)speechlen / (float)smpFreq;
 	  j_printf("%d samples (%.2f sec.)\n", speechlen, seclen);
 
-	  /* -rejectshort $B;XDj;~(B, $BF~NO$,;XDj;~4V0J2<$G$"$l$P(B
-	     $B$3$3$GF~NO$r4~5Q$9$k(B */
+	  /* -rejectshort , ϤְʲǤ
+	     ϤѤ */
 	  /* when using "-rejectshort", and input was shorter than
 	     specified, reject the input here */
 	  if (rejectshortlen > 0) {
 	    if (seclen * 1000.0 < rejectshortlen) {
 	      result_rejected("too short input");
-	      param = NULL;
 	      goto end_recog;
 	    }
 	  }
       
-	  /**********************************/
-	  /* acoustic analysis and encoding */
-	  /**********************************/
+	  /**********************************************/
+	  /* acoustic analysis and encoding of speech[] */
+	  /**********************************************/
 	  VERMES("### speech analysis (waveform -> MFCC)\n");
-	  /* subtract zero mean to remove DC offset */
-	  /* sub_zmean(speech, speechlen); */
-	  /* wav2mfcc() has no ZMEANSOURCE */
+	  /* CMN will be computed for the whole buffered input */
 	  param = new_wav2mfcc(speech, speechlen);
-	  if (param == NULL) continue;
+	  if (param == NULL) goto end_recog;
 
 	  /* output frame length */
 	  status_param(param);
@@ -415,8 +573,9 @@ main_recognition_loop()
 #ifdef SP_BREAK_CURRENT_FRAME
 	}
 #endif
-      }	/* end of raw speech data input and analysis */
-      
+      }	/* end of data input */
+      /* parameter has been got in 'param' */
+
       /********************************/
       /* check the analized parameter */
       /********************************/
@@ -427,7 +586,8 @@ main_recognition_loop()
 	selected_param = new_param_check_and_adjust(hmminfo, param, verbose_flag);
 	if (selected_param == NULL) { /* failed */
 	  free_param(param);
-	  continue;
+	  param = NULL;
+	  goto end_recog;
 	}
 	param = selected_param;
       }
@@ -436,6 +596,7 @@ main_recognition_loop()
       /******************************************************/
       /* 1st-pass --- backward search to compute heuristics */
       /******************************************************/
+      /* (for buffered speech input and HTK parameter file input) */
 #ifdef USE_NGRAM
       VERMES("### Recognition: 1st pass (LR beam with 2-gram)\n");
 #else
@@ -470,8 +631,13 @@ main_recognition_loop()
 
     end_1pass:
 
-      /* -rejectshort $B;XDj;~(B, $BF~NO$,;XDj;~4V0J2<$G$"$l$PC5:w<:GT$H$7$F(B */
-      /* $BBh#2%Q%9$r<B9T$;$:$K$3$3$G=*N;$9$k(B */
+      /**********************************/
+      /* end processing of the 1st-pass */
+      /**********************************/
+      /* on-the-fly 1st pass processing will join here */
+      
+      /* -rejectshort , ϤְʲǤõԤȤ */
+      /* 裲ѥ¹Ԥˤǽλ */
       /* when using "-rejectshort", and input was shorter than the specified
 	 length, terminate search here and output recognition failure */
       if (rejectshortlen > 0) {
@@ -482,8 +648,6 @@ main_recognition_loop()
 	}
       }
   
-
-      /* with realtime_flag, it joins at this point */
       /* if [-1pass] is specified, terminate search here */
       if (compute_only_1pass) {
 	goto end_recog;
@@ -499,16 +663,17 @@ main_recognition_loop()
       /* if terminate signal has been received, cancel this input */
       if (module_mode && module_wants_terminate()) goto end_recog;
 
+      /* if GMM is specified and result are to be rejected, terminate search here */
+      if (gmm_reject_cmn_string != NULL) {
+	if (! gmm_valid_input()) {
+	  result_rejected("by GMM");
+	  goto end_recog;
+	}
+      }
       /***********************************************/
       /* 2nd-pass --- forward search with heuristics */
       /***********************************************/
-#ifdef SCAN_BEAM
-      /* prepare score envelope for 2nd pass */
-      framemaxscore = (LOGPROB *)mymalloc(sizeof(LOGPROB)*backtrellis.framelen);
-      envl_init(backtrellis.framelen);
-#endif /* SCAN_BEAM */
-    
-#ifndef PASS2_STRICT_IWCD
+#if !defined(PASS2_STRICT_IWCD) || defined(FIX_35_PASS2_STRICT_SCORE)    
       /* adjust trellis score not to contain outprob of the last frames */
       bt_discount_pescore(wchmm, &backtrellis, param);
 #endif
@@ -520,9 +685,23 @@ main_recognition_loop()
 #endif
 
       /* execute stack-decoding search */
-      wchmm_fbs(param, &backtrellis, backmax, stack_size, nbest, hypo_overflow);
-#ifdef SCAN_BEAM
-      free(framemaxscore);
+#ifdef USE_NGRAM
+      wchmm_fbs(param, &backtrellis, backmax, stack_size, nbest, hypo_overflow, 0, 0);
+#else  /* USE_DFA */
+      if (multigramout_flag) {
+	/* execute 2nd pass multiple times for each grammar sequencially */
+	/* to output result for each grammar */
+	MULTIGRAM *m;
+	for(m = gramlist; m; m = m->next) {
+	  if (m->active) {
+	    j_printf("## search for gram #%d\n", m->id);
+	    wchmm_fbs(param, &backtrellis, backmax, stack_size, nbest, hypo_overflow, m->cate_begin, m->dfa->term_num);
+	  }
+	}
+      } else {
+	/* only the best among all grammar will be output */
+	wchmm_fbs(param, &backtrellis, backmax, stack_size, nbest, hypo_overflow, 0, dfa->term_num);
+      }
 #endif
 
     end_recog:
@@ -530,10 +709,17 @@ main_recognition_loop()
       /* end of recognition */
       /**********************/
 
+      /* update CMN info for next input (in case of realtime wave input) */
+      if (speech_input != SP_MFCFILE && realtime_flag && param != NULL) {
+	RealTimeCMNUpdate(param);
+      }
+
 #ifdef VISUALIZE
+      /* execute visualization */
       visual_show(&backtrellis);
 #endif
-	
+
+      /* free parameter work area */
       if (param != NULL) free_param(param);
 
       /* if needed, close the recording files */
@@ -552,18 +738,30 @@ main_recognition_loop()
 	VERMES("<<<restart the rest>>>\n\n");
 	param = rest_param;
       } else {
-	/* input has reached end of stream loop, terminate program */
-	if (ret <= 0) break;
+	/* input has reached end of stream, terminate program */
+	if (ret <= 0 && ret != -2) break;
       }
 #else
-      /* input has reached end of stream loop, terminate program */
-      if (ret <= 0) break;
+      /* input has reached end of stream, terminate program */
+      if (ret <= 0 && ret != -2) break;
 #endif
-    } /* end of stream loop */
+
+      /* recognition continues for next (silence-aparted) segment */
+      
+    } /* END OF STREAM LOOP */
+    
+    /* input stream ended. it will happen when
+       - input speech file has reached the end of file, 
+       - adinnet input has received end of segment mark from client,
+       - adinnet input has received end of input from client,
+       - adinnet client disconnected.
+    */
 
     if (speech_input != SP_MFCFILE) {
+      /* close the stream */
       adin_end();
     }
+    /* return to the opening of input stream */
   }
 
 }
@@ -572,6 +770,32 @@ main_recognition_loop()
 /* ------------------------------------------------------------------- */
 /* ------------- The Main Routine ------------------------------------ */
 /* ------------------------------------------------------------------- */
+/** 
+ * <JA>
+ * @brief  ᥤؿ
+ *
+ * Ԥʤ٥ȥ롼פƤӽФ
+ * ⥸塼⡼ɤΤȤ main_module_loop()̾Υɥ⡼
+ * ΤȤ main_recognition_loop() Ƥ֡
+ * 
+ * @param argc [in] ޥɰο
+ * @param argv [in] ޥɰ
+ * 
+ * @return ̾ｪλ 0 ֤
+ * </JA>
+ * <EN>
+ * @brief  Main application function.
+ *
+ * This function does some boot-time initialization, and calls event loop.
+ * main_module_loop() will be called when module mode, and
+ * main_recognition_loop() will be called when normal standalone mode.
+ * 
+ * @param argc [in] number of command argument
+ * @param argv [in] array of command argument
+ * 
+ * @return 0 on normal exit.
+ * </EN>
+ */
 int
 main(int argc, char *argv[])
 {
@@ -583,6 +807,12 @@ main(int argc, char *argv[])
   /* parse options and set variables */
   opt_parse(argc,argv,NULL);
 
+#ifdef CHARACTER_CONVERSION
+  if (j_printf_set_charconv(from_code, to_code) == FALSE) {
+    j_error("Error: character set conversion setup failed\n");
+  }
+#endif /* CHARACTER_CONVERSION */
+
   /* check option values */
   check_specs();
 
