@@ -1,79 +1,184 @@
+/**
+ * @file   adin-cut.c
+ * @author Akinobu LEE
+ * @date   Sat Feb 12 13:20:53 2005
+ *
+ * <JA>
+ * @brief  ߤӲָ
+ *
+ * ϥǥХβǡμߡӲ֤θФ
+ * Ԥʤޤ
+ *
+ * ֤θФϡ٥򺹿ѤƹԤʤäƤޤ
+ * ҤȤˡ٥뤷ͤۤ뿶ˤĤ򺹿򥫥Ȥ
+ * 줬ꤷʾˤʤСֳϸФȤ
+ * ߤ򳫻Ϥޤ򺹿ʲˤʤС
+ * ߤߤޤºݤˤϴڤФԤʤᡤ
+ * ˥ޡڤФޤ
+ * ޤɬפǤ DC offset ĴԤʤޤ
+ *
+ * ǡμߤ¹ԤϲνԤʤޤΤᡤ
+ * ǡϤμñ̡liveϤǤϰ֡ե
+ * ǤϥХåեˤȤˡȤƥХåؿƤФޤ
+ * ΥХåؿȤƥǡ¸ħС
+ * ʥե졼ƱΡǧʤؿꤷƤޤ
+ *
+ * ޥϤ NetAudio Ϥʤɤ Live Ϥľɤ߹硤
+ * ХåνŤϤ®٤ɤդʤȡ
+ * ǥХΥХåե졤ҤȤ礬ޤ
+ * Υ顼ɤˡ⤷¹ԴĶ pthread ѲǽǤС
+ * ߡָΤΩåɤȤưޤ
+ * ξ硤ΥåɤܥåɤȥХåե @a speech 𤷤ưʲΤ褦
+ * Ĵưޤ
+ * 
+ *    - Thread 1: ߡָХå
+ *        - ǥХ鲻ǡɤ߹ߤʤ鲻ָФԤʤ
+ *          Ф֤ΥץϥХåե @a speech ༡
+ *          ɲä롥
+ *        - ΥåɤϵưܥåɤΩư
+ *          嵭ưԤʤ³롥
+ *    - Thread 2: ǧԤʤܥå
+ *        - Хåե @a speech ֤Ȥ˴ƻ뤷ʥץ뤬
+ *          Thread 1 ˤäɲä줿餽λ
+ *          ʬХåեͤ롥
+ *
+ * ؿγפϰʲΤȤǤ
+ * JuliusΥᥤƤӽФؿ adin_go() Ǥ
+ * ߤȶָнΤ adin_cut() Ǥ
+ * ϥؤϡ adin_setup_func() оݤȤʤϥȥ꡼
+ * ϡɤ߹ߡߤδؿȤƸƤӽФȤǹԤʤޤ
+ * ޤڤФΤγƼѥ᡼ adin_setup_param() ǥåȤޤ
+ * </JA>
+ * <EN>
+ * @brief  Read in speech waveform and detect speech segment
+ *
+ * This file contains functions to get speech waveform from an audio device
+ * and detect speech segment.
+ *
+ * Speech detection is based on level threshold and zero cross count.
+ * The number of zero cross are counted for each incoming speech fragment.
+ * If the number becomes larger than specified threshold, the fragment
+ * is treated as a beginning of speech input (trigger on).  If the number goes
+ * below the threshold, the fragment will be treated as an
+ * end of speech input (trigger off).  In actual
+ * detection, margins are considered on the beginning and ending point, which
+ * will be treated as head and tail silence part.  DC offset normalization
+ * will be also performed if configured so.
+ *
+ * The triggered input speech data should be processed concurrently with the
+ * detection for real-time recognition.  For this purpose, after the
+ * beginning of speech input has been detected, the following triggered input
+ * fragments (samples of a certain period in live input, or buffer size in
+ * file input) are passed sequencially in turn to a callback function.
+ * The callback function should be specified by the caller, typicaly to
+ * store the recoded speech, or to process them into a frame-synchronous
+ * recognition process.
+ *
+ * When source is a live input such as microphone, the device buffer will
+ * overflow if the processing callback is slow.  In that case, some input
+ * fragments may be lost.  To prevent this, the A/D-in part together with
+ * speech detection will become an independent thread if @em pthread functions
+ * are supported.  The A/D-in and detection thread will cooperate with
+ * the original main thread through @a speech buffer, like the followings:
+ *
+ *    - Thread 1: A/D-in and speech detection thread
+ *        - reads audio input from source device and perform speech detection.
+ *          The detected fragments are immediately appended
+ *          to the @a speech buffer.
+ *        - will be detached after created, and run forever till the main
+ *          thread dies.
+ *    - Thread 2: Main thread
+ *        - performs speech processing and recognition.
+ *        - watches @a speech buffer, and if detect appendings of new samples
+ *          by the Thread 1, proceed the processing for the appended samples
+ *          and purge the finished samples from @a speech buffer.
+ *
+ * adin_setup_func() is used to switch audio input by specifying device-dependent
+ * open/read/close functions, and should be called at first.
+ * Function adin_setup_param() should be called after adin_setup_func() to
+ * set various parameters for speech detection.
+ * The adin_go() function is the top function that will be called from
+ * outside, to perform actual input processing.  adin_cut() is
+ * the main function to read audio input and detect speech segment.
+ * </EN>
+ *
+ * @sa adin.c
+ *
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2004 Kyoto University
- * Copyright (c) 2000-2004 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* adin-cut.c --- read audio input from device with skipping silence */
-
-/* $Id: adin-cut.c,v 1.20 2004/03/23 03:00:16 ri Exp $ */
-
-/* use zerocross & level threshold for silence detection */
-/* process one speech segment till next silence at a call */
-
-/* for real-time processing, it does not (basically) store speech samples
-   to a buffer.  Instead, an external function that directly processes the
-   incoming speech segment should be specified.  The incoming speech segments
-   are sequencially processed by the function. */
-
-#ifdef HAVE_PTHREAD
-/*
- *  threading is enabled.  Two threads share one buffer for sending/receiving
- *  incoming speech:
- *    Thread 1 (created): A/D-in thread with cutting
- *          read in speech data from device, detect sil, and if triggered,
- *          keep appending the segment to buffer.
- *    Thread 2 (original): processing thread
- *          keep watching the buffer, and process the incoming speech
- *          segment as soon as they appear.  Then it shrinks the buffer.
- *
- */
-#endif
-
 #include <sent/stddefs.h>
 #include <sent/speech.h>
 #include <sent/adin.h>
-
-#undef THREAD_DEBUG
-
-/* zero-cross params (local) */
-static int c_length  = 5000;	/* length of cycle buffer for zero-cross */
-static int c_offset  = 0;	/* data DC offset */
-static int wstep   = DEFAULT_WSTEP;	/* data window size */
-static int thres;		/* level threshold */
-static int noise_zerocross;	/* threshold of zerocross num in a period */
-static int nc_max;		/* tail margin in number of wstep segment */
-
-/* for delayed tail silence processing */
-static SP16 *swapbuf;		/* buffer for re-triggering in tail margin */
-static int sbsize, sblen;	/* size and current length of swapbuf[] */
-static int rest_tail;		/* unprocessed samples in swap buffer */
-
-/* device-dependent configuration */
-static boolean adin_cut_on;	/* if TRUE, do input segmentation by silence */
-static boolean silence_cut_default; /* device-dependent default value of adin_cut_on */
-static boolean strip_flag;	/* if TRUE, do invalid sample skipping */
-static boolean enable_thread = FALSE;	/* TRUE if device needs threading */
-static boolean ignore_speech_while_recog = TRUE; /* if TRUE, ignore speech input while recognition process */
-static boolean need_zmean;	/* if TRUE, do zmensource for incoming input */
-
-/* device handling function */
-static boolean (*ad_resume)();	/* restart input */
-static boolean (*ad_pause)();	/* stop input */
-static int (*ad_read)(SP16 *, int); /* read in samples */
-
 #ifdef HAVE_PTHREAD
-static void adin_thread_create(); /* create and start input thread */
+#include <pthread.h>
 #endif
 
-/* store device-dependent configurations to local */
-/* (only called from adin_select() via adin_register_func() ) */
+/* Define this if you want to output a debug message for threading */
+#undef THREAD_DEBUG
+
+/**
+ * @name Variables of zero-cross parameters and buffer sizes
+ * 
+ */
+//@{
+static int c_length  = 5000;	///< Computed length of cycle buffer for zero-cross, actually equals to head margin length
+static int c_offset  = 0;	///< Static data DC offset (obsolute, should be 0)
+static int wstep   = DEFAULT_WSTEP;	///< Data fragment size
+static int thres;		///< Input Level threshold (0-32767)
+static int noise_zerocross;	///< Computed threshold of zerocross num in the cycle buffer
+static int nc_max;		///< Computed number of fragments for tail margin
+//@}
+
+/**
+ * @name Variables for delayed tail silence processing
+ * 
+ */
+//@{
+static SP16 *swapbuf;		///< Buffer for re-triggering in tail margin
+static int sbsize, sblen;	///< Size and current length of @a swapbuf
+static int rest_tail;		///< Samples not processed yet in swap buffer
+//@}
+
+/**
+ * @name Work area for device configurations for local use
+ * 
+ */
+//@{
+static boolean (*ad_resume)();	///< Function pointer to (re)start input
+static boolean (*ad_pause)();	///< Function pointer to stop input
+static int (*ad_read)(SP16 *, int); ///< Function pointer to read in input samples
+static boolean adin_cut_on;	///< TRUE if do input segmentation by silence
+static boolean silence_cut_default; ///< Device-dependent default value of adin_cut_on()
+static boolean strip_flag;	///< TRUE if skip invalid zero samples
+static boolean enable_thread = FALSE;	///< TRUE if input device needs threading
+static boolean ignore_speech_while_recog = TRUE; ///< TRUE if ignore speech input between call, while waiting recognition process
+static boolean need_zmean;	///< TRUE if perform zmeansource
+//@}
+
+#ifdef HAVE_PTHREAD
+static void adin_thread_create(); ///< create and start A/D-in and detection thread 
+#endif
+
+/** 
+ * Store the given device-dependent functions and configuration values
+ * to local work area.  This function will be called from adin_select()
+ * via adin_register_func().
+ * 
+ */
 void
-adin_setup_func(int (*cad_read)(SP16 *, int),
-		boolean (*cad_pause)(),
-		boolean (*cad_resume)(),
-		boolean use_cut_def,
-		boolean need_thread
+adin_setup_func(int (*cad_read)(SP16 *, int), ///< [in] function to read input samples
+		boolean (*cad_pause)(), ///< [in] function to stop input
+		boolean (*cad_resume)(), ///< [in] function to (re-)start input
+		boolean use_cut_def, ///< [in] TRUE if the device needs speech segment detection by default
+		boolean need_thread ///< [in] TRUE if the device is live input and needs threading
 		)
 {
   ad_read = cad_read;
@@ -89,38 +194,46 @@ adin_setup_func(int (*cad_read)(SP16 *, int),
 #endif
 }
 
-/* setup silence detection parameters (should be called after adin_select()) */
+/** 
+ * Setup silence detection parameters (should be called after adin_select()).
+ * If using pthread, the A/D-in and detection thread will be started at the end
+ * of this function.
+ * 
+ * @param silence_cut [in] whether to perform silence cutting.
+ *                 0=force off, 1=force on, 2=keep device-specific default
+ * @param strip_zero [in] TRUE if enables stripping of zero samples 
+ * @param cthres [in]  input level threshold (0-32767)
+ * @param czc [in] zero-cross count threshold in a second
+ * @param head_margin [in] header margin length in msec
+ * @param tail_margin [in] tail margin length in msec
+ * @param sample_freq [in] sampling frequency: just providing value for computing other variables
+ * @param ignore_speech [in] TRUE if ignore speech input between call, while waiting recognition process
+ * @param need_zeromean [in] TRUE if perform zero-mean subtraction
+ */
 void
 adin_setup_param(int silence_cut, boolean strip_zero, int cthres, int czc, int head_margin, int tail_margin, int sample_freq, boolean ignore_speech, boolean need_zeromean)
 {
   float samples_in_msec;
-  /* whether to perform silence cutting */
-  /* 0...force off  1...force on  2...keep device-specific default */
   if (silence_cut < 2) {
     adin_cut_on = (silence_cut == 1) ? TRUE : FALSE;
   } else {
     adin_cut_on = silence_cut_default;
   }
-  /* whether to strip zero samples */
   strip_flag = strip_zero;
-  /* set level threshold */
   thres = cthres;
+  ignore_speech_while_recog = ignore_speech;
+  need_zmean = need_zeromean;
   /* calc & set internal parameter from configuration */
-  samples_in_msec = (float) sample_freq / 1000.0;
+  samples_in_msec = (float) sample_freq / (float)1000.0;
   /* cycle buffer length = head margin length */
-  c_length = head_margin * samples_in_msec;	/* in msec. */
+  c_length = (int)((float)head_margin * samples_in_msec);	/* in msec. */
   /* compute zerocross trigger count threshold in the cycle buffer */
   noise_zerocross = czc * c_length / sample_freq;
   /* process step */
   wstep = DEFAULT_WSTEP;
-  /* tail margin length (in wstep) */
-  nc_max = (tail_margin * samples_in_msec / wstep) + 2;
+  /* variables that comes from the tail margin length (in wstep) */
+  nc_max = (int)((float)(tail_margin * samples_in_msec / (float)wstep)) + 2;
   sbsize = tail_margin * samples_in_msec + (c_length * czc / 200);
-  /*printf("c_length=%d, noise_zerocross=%d nc_max=%d\n", c_length, noise_zerocross, nc_max);*/
-
-  ignore_speech_while_recog = ignore_speech;
-
-  need_zmean = need_zeromean;
 
 #ifdef HAVE_PTHREAD
   if (enable_thread) {
@@ -130,19 +243,32 @@ adin_setup_param(int silence_cut, boolean strip_zero, int cthres, int czc, int h
 #endif
 }
 
-/* query functions for the caller to know the resulting configurations */
+/** 
+ * Query function to check whether the input speech detection is on or off.
+ * 
+ * @return TRUE if on, FALSE if off.
+ */
 boolean
 query_segment_on()
 {
   return adin_cut_on;
 }
+
+/** 
+ * Query function to check whether the input threading is on or off.
+ * 
+ * @return TRUE if on, FALSE if off.
+ */
 boolean
 query_thread_on()
 {
   return enable_thread;
 }
 
-/* reset zero mean if required */
+/** 
+ * Reset zero mean data to re-estimate zero mean at the next input.
+ * 
+ */
 void
 adin_reset_zmean()
 {
@@ -151,23 +277,47 @@ adin_reset_zmean()
 
 
 #ifdef HAVE_PTHREAD
-#include <pthread.h>
-static pthread_t adin_thread;	/* thread info */
-static pthread_mutex_t mutex;	/* lock primitive */
-/* variables shared among adin thread and process thread */
-static boolean transfer_online = FALSE;	/* semaphore to start/stop recognition.  if TRUE, adin thread will store samples to speech[], and process thread will take them and process.  If FALSE, adin thread will get input and check trigger, but does not store them to speech[]  */
-static boolean adinthread_buffer_overflowed = FALSE; /* will be set to TRUE if speech[], where the adin thread will store triggered samples, has been overflowed */
+/**
+ * @name Variables related to POSIX threading
+ *
+ */
+//@{
+static pthread_t adin_thread;	///< Thread information
+static pthread_mutex_t mutex;	///< Lock primitive
+static SP16 *speech;		///< Unprocessed samples recorded by A/D-in thread
+static int speechlen;		///< Current length of @a speech
+/**
+ * @brief  Semaphore to start/stop recognition.
+ * 
+ * If TRUE, A/D-in thread will store incoming samples to @a speech and
+ * main thread will detect and process them.
+ * If FALSE, A/D-in thread will still get input and check trigger as the same
+ * as TRUE case, but does not store them to @a speech.
+ * 
+ */
+static boolean transfer_online = FALSE;	
+static boolean adinthread_buffer_overflowed = FALSE; ///< Will be set to TRUE if @a speech has been overflowed.
+//@}
 #endif
 
-/* temporary input data buffer */
-static SP16 *buffer = NULL;	/* temporary buffer to hold input samples */
-static int bpmax;		/* maximum length of buffer[] */
-static int bp;			/* current point to store data */
-static int current_len;		/* current length of stored samples */
-static SP16 *cbuf;		/* buffer for flushing cycle buffer just after detecting trigger  */
+/**
+ * @name Input data buffer
+ * 
+ */
+//@{
+static SP16 *buffer = NULL;	///< Temporary buffer to hold input samples
+static int bpmax;		///< Maximum length of @a buffer
+static int bp;			///< Current point to store the next data
+static int current_len;		///< Current length of stored samples
+static SP16 *cbuf;		///< Buffer for flushing cycle buffer just after detecting trigger 
+//@}
 
 
-/* purge processed samples in temporary buffer */
+/** 
+ * Purge samples already processed in the temporary buffer @a buffer.
+ * 
+ * @param from [in] Purge samples in range [0..from-1].
+ */
 static void
 adin_purge(int from)
 {
@@ -177,27 +327,31 @@ adin_purge(int from)
   bp = current_len - from;
 }
 
-/* main adin function */
-/* In threaded mode, this function will detach and loop forever in ad-in
-   thread,  storing triggered samples in speech[], and telling the status
-   to another process thread via boolean variable transfer_online.
-   The process thread, called from adin_go(), polls the length of speech[]
-   and transfer_online, and if there are stored samples, process it.
-   
-   In non-threaded mode, this function will be called directly from adin_go(),
-   and triggered samples are immediately processed within here.
+/** 
+ * @brief  Main A/D-in function
+ *
+ * In threaded mode, this function will detach and loop forever in ad-in
+ * thread, storing triggered samples in @a speech, and telling the status
+ * to another process thread via @a transfer_online.
+ * The process thread, called from adin_go(), polls the length of
+ * @a speech and @a transfer_online, and if there are stored samples,
+ * process them.
+ *
+ * In non-threaded mode, this function will be called directly from
+ * adin_go(), and triggered samples are immediately processed within here.
+ *
+ * In module mode, the function argument @a ad_check should be specified
+ * to poll the status of incoming command from client while recognition.
+ * 
+ * @return -1 on error, 0 on end of stream, >0 when paused by external process.
  */
-/* In module mode, the argument "ad_check" is specified to poll the status of
-   incoming command from client while recognition */
-/* return: -1 on error, 0 on end of stream, >0 paused by external process */
-
 static int
 adin_cut(
-	 int (*ad_process)(SP16 *, int), /* function to process the triggered samples */
-	 int (*ad_check)())	/* function periodically called while input processing */
+	 int (*ad_process)(SP16 *, int), ///< function to process the triggered samples
+	 int (*ad_check)())	///< function periodically called while input processing
 {
   static int i;
-  static boolean is_valid_data;	/* TRUE if we are now triggered */
+  static boolean is_valid_data;	///< TRUE if we are now triggered
   int ad_process_ret;
   int imax, len, cnt;
   static boolean end_of_stream;	/* will be set to TRUE if current input stream has reached the end (in case of file input or adinnet input).  If TRUE, no more input will be got by ad_read, but just process the already stored samples until it becomes empty */
@@ -330,7 +484,8 @@ adin_cut(
 	) {
       /* if ad_check() returns value < 0, termination of speech input is required */
       if ((i = (*ad_check)()) < 0) { /* -1: soft termination -2: hard termination */
-	if ((i == -1 && current_len == 0) || i == -2) {
+	//	if ((i == -1 && current_len == 0) || i == -2) {
+	if ((i == -1 && is_valid_data == FALSE) || i == -2) {
 	  end_status = -2;	/* recognition terminated by outer function */
 	  goto break_input;
 	}
@@ -712,14 +867,18 @@ break_input:
 /* threading functions */
 /***********************/
 
-/* adin-thread store data to speech[], and process thread read it */
-static SP16 *speech;		/* stored samples by adin-cut() */
-static int speechlen;		/* length of above */
-
 /*************************/
 /* adin thread functions */
 /*************************/
-/* call-back for storing triggered samples in adin thread */
+
+/** 
+ * Callback for storing triggered samples to @a speech in A/D-in thread.
+ * 
+ * @param now [in] triggered fragment
+ * @param len [in] length of above
+ * 
+ * @return always 0, to tell caller to continue recording.
+ */
 static int
 adin_store_buffer(SP16 *now, int len)
 {
@@ -742,14 +901,21 @@ adin_store_buffer(SP16 *now, int len)
   return(0);			/* continue */
 }
 
-/* A/D-in thread main function: only call adin_cut() with storing function */
+/** 
+ * A/D-in thread main function: just call adin_cut() with storing function.
+ * 
+ * @param dummy [in] a dummy data, not used.
+ */
 void
 adin_thread_input_main(void *dummy)
 {
   adin_cut(adin_store_buffer, NULL);
 }
 
-/* create A/D-in thread */
+/** 
+ * Start new A/D-in thread, and also initialize buffer @a speech.
+ * 
+ */
 static void
 adin_thread_create()
 {
@@ -776,6 +942,20 @@ adin_thread_create()
 /* process thread functions */
 /****************************/
 /* used for module mode: return value: -2 = input cancellation forced by control module */
+
+/** 
+ * @brief  Main function of processing triggered samples at main thread.
+ *
+ * Wait for the new samples to be stored in @a speech by A/D-in thread,
+ * and if found, process them.
+ * 
+ * @param ad_process [in] function to process the recorded fragments
+ * @param ad_check [in] function to be called periodically for checking
+ * incoming user command in module mode.
+ * 
+ * @return -2 when input terminated by result of the @a ad_check function,
+ * -1 on error, 0 on end of stream, >0 if successfully segmented.
+ */
 static int
 adin_thread_process(int (*ad_process)(SP16 *, int), int (*ad_check)())
 {
@@ -891,7 +1071,26 @@ adin_thread_process(int (*ad_process)(SP16 *, int), int (*ad_check)())
 #endif /* HAVE_PTHREAD */
 
 
-/* global entry point */
+
+
+/**
+ * @brief  Top function to start input processing
+ *
+ * If threading mode is enabled, this function simply enters to
+ * adin_thread_process() to process triggered samples detected by
+ * another running A/D-in thread.
+ *
+ * If threading mode is not available or disabled by either device requirement
+ * or OS capability, this function simply calls adin_cut() to detect speech
+ * segment from input device and process them concurrently by one process.
+ * 
+ * @param ad_process [in] function to process the recorded fragments
+ * @param ad_check [in] function to be called periodically for checking
+ * incoming user command in module mode.
+ * 
+ * @return the same as adin_thread_process() in threading mode, or
+ * same as adin_cut() when non-threaded mode.
+ */
 int
 adin_go(int (*ad_process)(SP16 *, int), int (*ad_check)())
 {
