@@ -1,19 +1,40 @@
+/**
+ * @file   rdhmmdef_regtree.c
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 02:30:28 2005
+ * 
+ * <JA>
+ * @brief  HTK %HMM եɤ߹ߡRegression tree
+ *
+ * Regression tree ¸줺ɤФޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Read HTK %HMM definition file: Regression tree
+ *
+ * The regression tree informations are not saved, just skipped.
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2004 Kyoto University
- * Copyright (c) 2000-2004 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* rdhmmdef_regtree.c --- read in hmmdefs (regression tree --- ignore) */
-
-/* $Id: rdhmmdef_regtree.c,v 1.4 2004/03/22 04:14:31 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 
-extern char *rdhmmdef_token;
+extern char *rdhmmdef_token;	///< Current token
 
-/* read in var data to skip till next macro */
+/** 
+ * Read in the regression tree to skip till next macro.
+ * 
+ * @param fp [in] file pointer
+ */
 static void
 regtree_read(FILE *fp)
 {
@@ -39,8 +60,13 @@ regtree_read(FILE *fp)
   }
 }
 
-
-/* define regtree data */
+/** 
+ * Skip a regression tree data or its macro reference.
+ * 
+ * @param name [in] macro name
+ * @param fp [in] file pointer
+ * @param hmm [in] %HMM definition data
+ */
 void
 def_regtree_macro(char *name, FILE *fp, HTK_HMM_INFO *hmm)
 {
