@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2003 NAIST
+ * Copyright (c) 2003-2005 Kawahara Lab., Kyoto University 
+ * Copyright (c) 2003-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -53,13 +55,13 @@ main(int argc, char *argv[])
   j_printf("---- writing ----\n");
   j_printf("filename: %s\n", argv[2]);
   
-  if ((fp = fopen(argv[2], "w")) == NULL) {
+  if ((fp = fopen_writefile(argv[2])) == NULL) {
     j_error("failed to open %s for writing\n", argv[2]);
   }
   if (write_binhmm(fp, hmminfo) == FALSE) {
     j_error("failed to write to %s\n", argv[2]);
   }
-  if (fclose(fp) != 0) {
+  if (fclose_writefile(fp) != 0) {
     j_error("failed to close %s\n", argv[2]);
   }
 
